#pragma once
#include "B4RDefines.h"
#include "PID_v1.h"
//  Constants used in some of the functions below
//~version: 1.00
//~DependsOn rpid_defines.h

namespace B4R {
	//~shortname: PID
	
	class B4RPID {
		private:
			uint8_t backend[sizeof(PID)];
			PID* rpid;
		public:
			// * constructor.  links the PID to the Input, Output, and Setpoint.  Initial tuning parameters are also set here.
			// (overload for specifying proportional mode)
			void Initialize(Double Input, Double Output, Double Setpoint, Double Kp, Double Ki, Double Kd, Byte POn, Byte ControllerDirection);
			
			//To allow backwards compatability for v1.1, or for people that just want
            //to use Proportional on Error without explicitly saying so
            //*********Not Implemented for B4R ************************************************
			//void Initialize(Input, Output, Setpoint, Kp, Ki, Kd, P_ON_E, ControllerDirection);
			
			// * sets PID to either Manual (0) or Auto (non-0)
			void SetMode(Byte Mode);
			
			// * performs the PID calculation. it should be called every time loop() cycles. ON/OFF and
            //   calculation frequency can be set using SetMode - SetSampleTime respectively
			bool Compute();
			
			// * clamps the output to a specific range. 0-255 by default, but
			//   it's likely the user will want to change this depending on the application
			void SetOutputLimits(Double Min, Double Max);

			// * While most users will set the tunings once in the constructor, this function gives the user the option
            //   of changing tunings during runtime for Adaptive control
			void SetTunings(Double Kp, Double Ki, Double Kd);
			
			// * overload for specifying proportional mode
			void SetTunings(Double Kp, Double Ki, Double Kd, Byte POn);
			
			// * Sets the Direction, or "Action" of the controller. DIRECT means the output will increase when error is positive. REVERSE
			//   means the opposite.  it's very unlikely that this will be needed once it is set in the constructor.
			void SetControllerDirection(Byte Direction);

			// * sets the frequency, in Milliseconds, with which the PID calculation is performed.  default is 100
			void SetSampleTime(Byte NewSampleTime);
			
			// These functions query the pid for interal values. they were created mainly for the pid front-end,
			//  where it's important to know what is actually inside the PID.
			
			Double GetKp();
			Double GetKi();
			Double GetKd();
			Byte GetMode();
			Byte GetDirection();
			
			void Initialize();
	};	
	Double dispKp;				// * we'll hold on to the tuning parameters in user-entered 
	Double dispKi;				//   format for display purposes
	Double dispKd;				//
    
	Double kp;                  // * (P)roportional Tuning Parameter
	Double ki;                  // * (I)ntegral Tuning Parameter
	Double kd;                  // * (D)erivative Tuning Parameter

	Byte controllerDirection;
	Byte pOn;

//	Double *myInput;              // * Pointers to the Input, Output, and Setpoint variables
//	Double *myOutput;             //   This creates a hard link between the variables and the 
//	Double *mySetpoint;           //   PID, freeing the user from having to constantly tell u 
                                  //   what these values are.  with pointers we'll just know.
			  
	Long lastTime;
	Double outputSum, lastInput;

	Long SampleTime;
	Double outMin, outMax;
	bool inAuto, pOnE;
	
}			
//	Double dispKp;				// * we'll hold on to the tuning parameters in user-entered 
//	Double dispKi;				//   format for display purposes
//	Double dispKd;				//
    
//	Double kp;                  // * (P)roportional Tuning Parameter
//  Double ki;                  // * (I)ntegral Tuning Parameter
//  Double kd;                  // * (D)erivative Tuning Parameter

//	Int controllerDirection;
//	Int pOn;

//  Double *myInput;              // * Pointers to the Input, Output, and Setpoint variables
//  Double *myOutput;             //   This creates a hard link between the variables and the 
//  Double *mySetpoint;           //   PID, freeing the user from having to constantly tell us
                                  //   what these values are.  with pointers we'll just know.
			  
//	Long lastTime;
//	Double outputSum, lastInput;

//	Long SampleTime;
//	Double outMin, outMax;
//	bool inAuto, pOnE;

